import ts from "typescript";
import { FullConfig } from "./types";
export declare type StoreInfo = {
    filePath?: string;
    name?: string;
    /**
     * The name of the class that extends Store
     */
    className?: string;
    /**
     * If the dev defined `public static vueBinding = 'x';` in their store, this will hold that value
     */
    vueBinding?: string;
    /**
     * This will hold the name of our store export `export const store = new MyStore();`
     */
    exportName?: string;
};
export declare class Context {
    program: ts.Program;
    filePaths: string[];
    modules: {
        module: ts.SourceFile;
        store: StoreMeta;
    }[];
    stores: StoreMeta[];
    config: FullConfig;
    compilerHost: ts.CompilerHost;
    printer: ts.Printer;
    typeChecker: ts.TypeChecker;
    static instance: Context;
    private parsedCommandLine;
    init(pluginConfig: FullConfig): void;
    setFilePaths(): string[];
    reloadModules(): void;
    process(): void;
    writeFiles(): void;
    rebuild(): void;
    private processModule;
    isStoreFile(path: string): boolean;
    getStoreByFilePath(path: string): StoreMeta | undefined;
}
export declare class StoreMeta {
    /**
     * The absolute path to the store file
     */
    absFilePath: string;
    /**
     * The relative path of the store file, for ex, if importing in /src/stores/, and our file is at /src/stores/MyStore.ts, this will be MyStore.ts
     */
    relStoreFilePath: string;
    /**
     * The name of the store file
     */
    name: string;
    /**
     * The name of the class that extends Store
     */
    className?: string;
    /**
     * This will hold the name of our store export `export const store = new MyStore();`
     */
    exportName?: string;
    /**
     * If the dev defined `public static vueBinding = 'x';` in their store, this will hold that value
     */
    vueBinding?: string;
    actions: ActionMeta[];
    constructor(file: ts.SourceFile, absStoresPath: string);
    isValid(): boolean;
    finalize(): this;
    private formatVueBinding;
}
declare class ActionMeta {
    name: string;
    params: any[];
    private method;
    private signature;
    constructor(method: ts.MethodDeclaration, name: string);
    addParam(param: ts.ParameterDeclaration): void;
    toJSON(): {
        name: string;
        params: any[];
    };
}
export {};
